from network_ff import Nn
import csv
import random
import numpy as np
from matplotlib import pyplot as plt
from datetime import datetime

def find_best_network_params(training_path, case = 1):
    accuracies = []

    for overall_i in range(3):
        training_set = []
        val_set = []

        training_ids = []

        with open(training_path) as f:
            reader = csv.reader(f)

            for row in reader:
                if row[0] not in training_ids:
                    training_ids.append(row[0])

            random.shuffle(training_ids)

            val_ids = training_ids[:int(len(training_ids)*0.2)]

            f.seek(0)

            for row in reader:
                if case == 1 or case == 4:
                    input = row[1:-1]
                elif case == 2 or case == 5:
                    input = row[1:5] + row[-2:-1]
                elif case == 3 or case == 6:
                    input = row[5:-1]

                if row[-1] == "home":
                    if row[0] in val_ids:
                        val_set.append([input, [1,0,0]])
                    else:
                        training_set.append([input, [1,0,0]])
                elif row[-1] == "none":
                    if row[0] in val_ids:
                        val_set.append([input, [0,1,0]])
                    else:
                        training_set.append([input, [0,1,0]])
                elif row[-1] == "away":
                    if row[0] in val_ids:
                        val_set.append([input, [0,0,1]])
                    else:
                        training_set.append([input, [0,0,1]])

        random.shuffle(training_set)
        random.shuffle(val_set)

        input_size = len(training_set[0][0])

        setups = []
        learning_rates = [0.0005 ,0.001, 0.002]
        hidden_layers = [[int((input_size+3)/2)], [input_size], [input_size, input_size], [int((input_size+3)/2), int((input_size+3)/2)], [int((input_size+3)/2), int((input_size+3)/2), int((input_size+3)/2)]]
        hidden_activations = ["tanh", "relu", "sigmoid"]
        optimizers = ["adam"]
        batch_sizes = [32, 64, 128, 256, 512]

        for learning_rate in learning_rates:
            for hidden_layer in hidden_layers:
                for hidden_activation in hidden_activations:
                    for optimizer in optimizers:
                        for batch_size in batch_sizes:
                            setups.append({
                                "learning_rate": learning_rate,
                                "hidden_layer":hidden_layer,
                                "hidden_activation":hidden_activation,
                                "optimizer":optimizer,
                                "batch_size":batch_size
                            })


        best_acc_overall = 0
        best_setup = 0

        accuracies_this_run = []
        parameters_list = []
        for i, setup in enumerate(setups):
            nn = Nn(input_size=input_size, output_size=3, learning_rate=setup["learning_rate"], hidden_layers=setup["hidden_layer"],
                     hidden_activation=setup["hidden_activation"], optimizer=setup["optimizer"], output_activation="softmax",
                     epochs=30, batch_size=setup["batch_size"], from_file=None)
            best_acc, epoch_saved = nn.train(training_set, val_set, "C:/Users/Mii Erik/OneDrive/Dokumenter/Skole/Masteroppgave/master/nets/" + str(i) + ".h5")
            if best_acc_overall < best_acc:
                best_acc_overall = best_acc
                best_setup = setup

            accuracies_this_run.append(best_acc)
            parameters_list.append(setup)
        accuracies.append([accuracies_this_run, parameters_list])

    avg_accuracies = []

    for indx, _ in enumerate(accuracies[0][0]):
        avg_acc = (accuracies[0][0][indx] + accuracies[1][0][indx] + accuracies[2][0][indx])/3
        parameters = accuracies[0][1][indx]
        avg_accuracies.append([avg_acc, parameters])

    avg_accuracies = sorted(avg_accuracies, key=lambda x: x[0], reverse = True)

    print("\n")
    for each in avg_accuracies:
        print(each)


def betting_sim_1(training_path, testdata_path, hidden_layer, hidden_activation, l_rate, batch_size, case = 1):
    bankroll_array_kelly = []
    bankroll_array_fix_return=[]
    bankroll_array_fixed_bet = []
    bankroll_array_rue = []
    for i in range (5):
        training_set = []
        val_set = []
        training_ids = []

        with open(training_path) as f:
            reader = csv.reader(f)

            for row in reader:
                if row[0] not in training_ids:
                    training_ids.append(row[0])

            random.shuffle(training_ids)

            val_ids = training_ids[:int(len(training_ids) * 0.2)]

            f.seek(0)

            for row in reader:
                if case == 1 or case == 4:
                    input = row[1:-1]
                elif case == 2 or case == 5:
                    input = row[1:5] + row[-2:-1]
                elif case == 3 or case == 6:
                    input = row[5:-1]

                if row[-1] == "home":
                    if row[0] in val_ids:
                        val_set.append([input, [1, 0, 0]])
                    else:
                        training_set.append([input, [1, 0, 0]])
                elif row[-1] == "none":
                    if row[0] in val_ids:
                        val_set.append([input, [0, 1, 0]])
                    else:
                        training_set.append([input, [0, 1, 0]])
                elif row[-1] == "away":
                    if row[0] in val_ids:
                        val_set.append([input, [0, 0, 1]])
                    else:
                        training_set.append([input, [0, 0, 1]])

        random.shuffle(training_set)
        random.shuffle(val_set)

        input_size = len(training_set[0][0])

        nn = Nn(input_size=input_size, output_size=3, learning_rate=l_rate,
                hidden_layers=hidden_layer,
                hidden_activation=hidden_activation, optimizer="adam", output_activation="softmax",
                epochs=30, batch_size=batch_size, from_file=None)
        nn.train(training_set, val_set, "C:/Users/Mii Erik/OneDrive/Dokumenter/Skole/Masteroppgave/master/nets/betting_sim_case" + str(case)+ "_" + str(i)+ ".h5")

        matches = []
        test_path = testdata_path
        with open(test_path) as f:
            reader = csv.reader(f)

            for row in reader:
                input = row[1:-4]
                if case == 1 or case == 4:
                    input = row[1:-4]
                elif case == 2 or case == 5:
                    input = row[1:5] + row[-5:-4]
                elif case == 3 or case == 6:
                    input = row[5:-4]

                if row[-1] == "home":
                    matches.append([input, [1, 0, 0], row[-4], row[-3], row[0]])
                elif row[-1] == "none":
                    matches.append([input, [0, 1, 0], row[-4], row[-3], row[0]])
                elif row[-1] == "away":
                    matches.append([input, [0, 0, 1], row[-4], row[-3], row[0]])

        sorted_matches = sorted(matches, key=lambda x: datetime.strptime(x[2], "%Y-%m-%d %H:%M:%S"), reverse=False)

        predictions = nn.run_for_betting(sorted_matches)

        odds_set = {}
        id = "000"
        odds_for_id = []

        with open("odds/uf_lo_export_nextgoal.txt", "r") as oddsfile:
            filereader = csv.reader(oddsfile)

            next(filereader)

            for row in filereader:
                data = row[0].split('\t')
                if id != data[1]:
                    odds_set[id] = odds_for_id
                    id = data[1]
                    odds_for_id = []
                odds_for_id.append(data)
            odds_set[id] = odds_for_id

        #Fixed bet
        bankroll_array_fixed_bet_inner = []
        total_units_bettet = 0
        bankroll = 100
        top_unit = 0
        corrects = 0
        total_odds = 0
        total_bets_taken = 0
        bankroll_array_fixed_bet_inner.append(bankroll)

        for j, match in enumerate(sorted_matches):
            match_id = match[-1]
            placing_time = datetime.strptime(match[2], "%Y-%m-%d %H:%M:%S")
            deciding_time = datetime.strptime(match[3], "%Y-%m-%d %H:%M:%S")

            prediction = predictions[j]

            odds_for_match = odds_set[match_id]
            odds_data = odds_for_match[0]
            for each_update in odds_for_match:
                time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
                if time > placing_time:
                    break
                odds_data = each_update

            odds = [0.97 / float(odds_data[5]), 0.97 / float(odds_data[7]), 0.97 / float(odds_data[9])]

            index_highest_prob = np.argmax(prediction)
            units_to_bet = 1

            expected_value = odds[index_highest_prob]*prediction[index_highest_prob] - 1

            if odds[index_highest_prob] <= 1:
                bankroll_array_fixed_bet_inner.append(bankroll)
                continue

            if expected_value <= 0:
                bankroll_array_fixed_bet_inner.append(bankroll)
                continue

            total_units_bettet += units_to_bet
            total_bets_taken += 1
            bankroll -= units_to_bet

            if units_to_bet > top_unit:
                top_unit = units_to_bet

            if index_highest_prob == 0:
                if match[1] == [1, 0, 0]:
                    total_odds += odds[index_highest_prob]
                    reward = units_to_bet * odds[index_highest_prob]
                    bankroll += reward
                    corrects += 1

            elif index_highest_prob == 1:
                if match[1] == [0, 1, 0]:
                    reward = units_to_bet * odds[index_highest_prob]
                    bankroll += reward
                    corrects += 1
                    total_odds += odds[index_highest_prob]

            else:
                if match[1] == [0, 0, 1]:
                    reward = units_to_bet * odds[index_highest_prob]
                    bankroll += reward
                    corrects += 1
                    total_odds += odds[index_highest_prob]

            bankroll_array_fixed_bet_inner.append(bankroll)

        print("\nYou have this left after the fixed-bet simulation:", bankroll)
        print("ROI:", ((bankroll - 100) / total_units_bettet) * 100)
        bankroll_array_fixed_bet.append([bankroll_array_fixed_bet_inner, ((bankroll - 100) / total_units_bettet) * 100])

        #Fixed return
        bankroll_array_fix_return_inner = []
        total_units_bettet = 0
        bankroll = 100
        top_unit = 0
        corrects = 0
        total_odds = 0
        total_bets_taken = 0
        bankroll_array_fix_return_inner.append(bankroll)

        for j, match in enumerate(sorted_matches):
            match_id = match[-1]
            placing_time = datetime.strptime(match[2], "%Y-%m-%d %H:%M:%S")
            deciding_time = datetime.strptime(match[3], "%Y-%m-%d %H:%M:%S")

            prediction = predictions[j]

            odds_for_match = odds_set[match_id]
            odds_data = odds_for_match[0]
            for each_update in odds_for_match:
                time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
                if time > placing_time:
                    break
                odds_data = each_update

            odds = [0.97 / float(odds_data[5]), 0.97 / float(odds_data[7]), 0.97 / float(odds_data[9])]

            index_highest_prob = np.argmax(prediction)
            units_to_bet = 1/odds[index_highest_prob]

            expected_value = odds[index_highest_prob]*prediction[index_highest_prob] - 1

            if odds[index_highest_prob] <= 1:
                bankroll_array_fix_return_inner.append(bankroll)
                continue

            if expected_value <= 0:
                bankroll_array_fix_return_inner.append(bankroll)
                continue

            total_units_bettet += units_to_bet
            total_bets_taken += 1
            bankroll -= units_to_bet

            if units_to_bet > top_unit:
                top_unit = units_to_bet

            if index_highest_prob == 0:
                if match[1] == [1, 0, 0]:
                    total_odds += odds[index_highest_prob]
                    reward = units_to_bet * odds[index_highest_prob]
                    bankroll += reward
                    corrects += 1

            elif index_highest_prob == 1:
                if match[1] == [0, 1, 0]:
                    reward = units_to_bet * odds[index_highest_prob]
                    bankroll += reward
                    corrects += 1
                    total_odds += odds[index_highest_prob]

            else:
                if match[1] == [0, 0, 1]:
                    reward = units_to_bet * odds[index_highest_prob]
                    bankroll += reward
                    corrects += 1
                    total_odds += odds[index_highest_prob]

            bankroll_array_fix_return_inner.append(bankroll)

        print("\nYou have this left after the fixed-return simulation:", bankroll)
        print("ROI:", ((bankroll - 100) / total_units_bettet) * 100)
        bankroll_array_fix_return.append(
            [bankroll_array_fix_return_inner, ((bankroll - 100) / total_units_bettet) * 100])

        #Kelly
        bankroll_array_kelly_inner = []
        total_units_bettet = 0
        bankroll = 100
        top_unit = 0
        corrects = 0
        total_odds = 0
        total_bets_taken = 0
        bankroll_array_kelly_inner.append(bankroll)

        for j, match in enumerate(sorted_matches):
            match_id = match[-1]
            placing_time = datetime.strptime(match[2], "%Y-%m-%d %H:%M:%S")
            deciding_time = datetime.strptime(match[3], "%Y-%m-%d %H:%M:%S")

            prediction = predictions[j]

            odds_for_match = odds_set[match_id]
            odds_data = odds_for_match[0]
            for each_update in odds_for_match:
                time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
                if time > placing_time:
                    break
                odds_data = each_update

            odds = [0.97 / float(odds_data[5]), 0.97 / float(odds_data[7]), 0.97 / float(odds_data[9])]

            index_highest_prob = np.argmax(prediction)
            fraction=0.25
            units_to_bet = fraction*bankroll*((prediction[index_highest_prob]*odds[index_highest_prob] - 1)/(odds[index_highest_prob] - 1))

            expected_value = odds[index_highest_prob]*prediction[index_highest_prob] - 1

            if odds[index_highest_prob] <= 1:
                bankroll_array_kelly_inner.append(bankroll)
                continue

            if expected_value <= 0:
                bankroll_array_kelly_inner.append(bankroll)
                continue

            if units_to_bet <= 0:
                bankroll_array_kelly_inner.append(bankroll)
                continue
            total_units_bettet += units_to_bet
            total_bets_taken += 1
            bankroll -= units_to_bet

            if units_to_bet > top_unit:
                top_unit = units_to_bet

            if index_highest_prob == 0:
                if match[1] == [1, 0, 0]:
                    total_odds += odds[index_highest_prob]
                    reward = units_to_bet * odds[index_highest_prob]
                    bankroll += reward
                    corrects += 1
            elif index_highest_prob == 1:
                if match[1] == [0, 1, 0]:
                    reward = units_to_bet * odds[index_highest_prob]
                    bankroll += reward
                    corrects += 1
                    total_odds += odds[index_highest_prob]

            else:
                if match[1] == [0, 0, 1]:
                    reward = units_to_bet * odds[index_highest_prob]
                    bankroll += reward
                    corrects += 1
                    total_odds += odds[index_highest_prob]

            bankroll_array_kelly_inner.append(bankroll)


        print("\nYou have this left after the kelly simulation:", bankroll)
        print("ROI:", ((bankroll - 100) / total_units_bettet) * 100)
        bankroll_array_kelly.append(
            [bankroll_array_kelly_inner, ((bankroll - 100) / total_units_bettet) * 100])

        #Variance-adjusted
        bankroll_array_rue_inner = []
        total_units_bettet = 0
        bankroll = 100
        top_unit = 0
        corrects = 0
        total_odds = 0
        total_bets_taken = 0
        bankroll_array_rue_inner.append(bankroll)

        for j, match in enumerate(sorted_matches):
            match_id = match[-1]
            placing_time = datetime.strptime(match[2], "%Y-%m-%d %H:%M:%S")
            deciding_time = datetime.strptime(match[3], "%Y-%m-%d %H:%M:%S")

            prediction = predictions[j]

            odds_for_match = odds_set[match_id]
            odds_data = odds_for_match[0]
            for each_update in odds_for_match:
                time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
                if time > placing_time:
                    break
                odds_data = each_update

            odds = [0.97 / float(odds_data[5]), 0.97 / float(odds_data[7]), 0.97 / float(odds_data[9])]

            index_highest_prob = np.argmax(prediction)
            fraction = 0.25

            units_to_bet = (1)/((2*odds[index_highest_prob])*(1-prediction[index_highest_prob]))

            expected_value = odds[index_highest_prob] * prediction[index_highest_prob] - 1

            if odds[index_highest_prob] <= 1:
                bankroll_array_rue_inner.append(bankroll)
                continue

            if expected_value <= 0:
                bankroll_array_rue_inner.append(bankroll)
                continue

            if units_to_bet <= 0:
                bankroll_array_rue_inner.append(bankroll)
                continue

            total_units_bettet += units_to_bet
            total_bets_taken += 1
            bankroll -= units_to_bet

            if units_to_bet > top_unit:
                top_unit = units_to_bet

            if index_highest_prob == 0:
                if match[1] == [1, 0, 0]:
                    total_odds += odds[index_highest_prob]
                    reward = units_to_bet * odds[index_highest_prob]
                    bankroll += reward
                    corrects += 1
            elif index_highest_prob == 1:
                if match[1] == [0, 1, 0]:
                    reward = units_to_bet * odds[index_highest_prob]
                    bankroll += reward
                    corrects += 1
                    total_odds += odds[index_highest_prob]

            else:
                if match[1] == [0, 0, 1]:
                    reward = units_to_bet * odds[index_highest_prob]
                    bankroll += reward
                    corrects += 1
                    total_odds += odds[index_highest_prob]

            bankroll_array_rue_inner.append(bankroll)

        print("\nYou have this left after the variance-adjusted simulation:", bankroll)
        print("ROI:", ((bankroll - 100) / total_units_bettet) * 100)
        bankroll_array_rue.append(
            [bankroll_array_rue_inner, ((bankroll - 100) / total_units_bettet) * 100])

    bankroll_array_kelly_avg = []
    bankroll_array_fix_return_avg = []
    bankroll_array_fixed_bet_avg = []
    bankroll_array_rue_avg = []


    bankroll_array_kelly_best = bankroll_array_kelly[0][0]
    bankroll_array_kelly_best_roi = bankroll_array_kelly[0][1]
    bankroll_array_fix_return_best = bankroll_array_fix_return[0][0]
    bankroll_array_fix_return_best_roi = bankroll_array_fix_return[0][1]
    bankroll_array_fixed_bet_best = bankroll_array_fixed_bet[0][0]
    bankroll_array_fixed_bet_best_roi = bankroll_array_fixed_bet[0][1]
    bankroll_array_rue_best = bankroll_array_rue[0][0]
    bankroll_array_rue_best_roi = bankroll_array_rue[0][1]

    bankroll_array_kelly_worst = bankroll_array_kelly[0][0]
    bankroll_array_kelly_worst_roi = bankroll_array_kelly[0][1]
    bankroll_array_fix_return_worst = bankroll_array_fix_return[0][0]
    bankroll_array_fix_return_worst_roi = bankroll_array_fix_return[0][1]
    bankroll_array_fixed_bet_worst = bankroll_array_fixed_bet[0][0]
    bankroll_array_fixed_bet_worst_roi = bankroll_array_fixed_bet[0][1]
    bankroll_array_rue_worst = bankroll_array_rue[0][0]
    bankroll_array_rue_worst_roi = bankroll_array_rue[0][1]

    for j, _ in enumerate(bankroll_array_kelly[0][0]):
        bankroll_array_kelly_avg.append((bankroll_array_kelly[0][0][j] + bankroll_array_kelly[1][0][j]
                                         + bankroll_array_kelly[2][0][j] + bankroll_array_kelly[3][0][j]
                                         + bankroll_array_kelly[4][0][j]) / 5)

        bankroll_array_fix_return_avg.append(
            (bankroll_array_fix_return[0][0][j] + bankroll_array_fix_return[1][0][j]
             + bankroll_array_fix_return[2][0][j] + bankroll_array_fix_return[3][0][j]
             + bankroll_array_fix_return[4][0][j]) / 5)

        bankroll_array_fixed_bet_avg.append(
            (bankroll_array_fixed_bet[0][0][j] + bankroll_array_fixed_bet[1][0][j]
             + bankroll_array_fixed_bet[2][0][j] + bankroll_array_fixed_bet[3][0][j]
             + bankroll_array_fixed_bet[4][0][j]) / 5)

        bankroll_array_rue_avg.append(
            (bankroll_array_rue[0][0][j] + bankroll_array_rue[1][0][j]
             + bankroll_array_rue[2][0][j] + bankroll_array_rue[3][0][j]
             + bankroll_array_rue[4][0][j]) / 5)

    avg_roi_kelly = 0
    avg_roi_rue = 0
    avg_roi_fixed_bet = 0
    avg_roi_fixed_return = 0

    for i in range(5):
        avg_roi_kelly += bankroll_array_kelly[i][1]
        avg_roi_rue += bankroll_array_rue[i][1]
        avg_roi_fixed_bet += bankroll_array_fixed_bet[i][1]
        avg_roi_fixed_return += bankroll_array_fix_return[i][1]

        if bankroll_array_rue[i][1] > bankroll_array_rue_best_roi:
            bankroll_array_rue_best = bankroll_array_rue[i][0]
            bankroll_array_rue_best_roi = bankroll_array_rue[i][1]

        if bankroll_array_fixed_bet[i][1] > bankroll_array_fixed_bet_best_roi:
            bankroll_array_fixed_bet_best = bankroll_array_fixed_bet[i][0]
            bankroll_array_fixed_bet_best_roi = bankroll_array_fixed_bet[i][1]

        if bankroll_array_fix_return[i][1] > bankroll_array_fix_return_best_roi:
            bankroll_array_fix_return_best = bankroll_array_fix_return[i][0]
            bankroll_array_fix_return_best_roi = bankroll_array_fix_return[i][1]

        if bankroll_array_kelly[i][1] > bankroll_array_kelly_best_roi:
            bankroll_array_kelly_best = bankroll_array_kelly[i][0]
            bankroll_array_kelly_best_roi = bankroll_array_kelly[i][1]



        if bankroll_array_rue[i][1] < bankroll_array_rue_worst_roi:
            bankroll_array_rue_worst = bankroll_array_rue[i][0]
            bankroll_array_rue_worst_roi = bankroll_array_rue[i][1]

        if bankroll_array_fixed_bet[i][1] < bankroll_array_fixed_bet_worst_roi:
            bankroll_array_fixed_bet_worst = bankroll_array_fixed_bet[i][0]
            bankroll_array_fixed_bet_worst_roi = bankroll_array_fixed_bet[i][1]

        if bankroll_array_fix_return[i][1] < bankroll_array_fix_return_worst_roi:
            bankroll_array_fix_return_worst = bankroll_array_fix_return[i][0]
            bankroll_array_fix_return_worst_roi = bankroll_array_fix_return[i][1]

        if bankroll_array_kelly[i][1] < bankroll_array_kelly_worst_roi:
            bankroll_array_kelly_worst = bankroll_array_kelly[i][0]
            bankroll_array_kelly_worst_roi = bankroll_array_kelly[i][1]

    avg_roi_kelly /= 5
    avg_roi_rue /= 5
    avg_roi_fixed_bet /= 5
    avg_roi_fixed_return /= 5

    print("\n\nFixed bet avg:", avg_roi_fixed_bet, ". Min:", bankroll_array_fixed_bet_worst_roi,". Max:", bankroll_array_fixed_bet_best_roi)
    print("Fixed return avg:", avg_roi_fixed_return, ". Min:", bankroll_array_fix_return_worst_roi, ". Max:", bankroll_array_fix_return_best_roi)
    print("Kelly avg:", avg_roi_kelly, ". Min:", bankroll_array_kelly_worst_roi, ". Max:", bankroll_array_kelly_best_roi)
    print("Variance avg:", avg_roi_rue, ". Min:", bankroll_array_rue_worst_roi, ". Max:", bankroll_array_rue_best_roi)

    label_for_plot = []
    for j, _ in enumerate(bankroll_array_fixed_bet[0][0]):
        label_for_plot.append(j)
    plt.figure()
    plt.plot(label_for_plot, bankroll_array_fixed_bet_worst, 'r-', label='Min')
    plt.plot(label_for_plot, bankroll_array_fixed_bet_best, 'g-', label='Max')
    plt.plot(label_for_plot, bankroll_array_fixed_bet_avg, 'y-', label='Avg')
    plt.axhline(y=100, color='k', linestyle='-')
    plt.xlabel("Match")
    plt.ylabel("Bankroll")
    plt.legend(loc=0, ncol=2)
    plt.title("Case " + str(case) + " - Fixed bet")

    plt.figure()
    plt.plot(label_for_plot, bankroll_array_fix_return_worst, 'r-', label='Min')
    plt.plot(label_for_plot, bankroll_array_fix_return_best, 'g-', label='Max')
    plt.plot(label_for_plot, bankroll_array_fix_return_avg, 'y-', label='Avg')
    plt.axhline(y=100, color='k', linestyle='-')
    plt.xlabel("Match")
    plt.ylabel("Bankroll")
    plt.legend(loc=0, ncol=2)
    plt.title("Case " + str(case) + " - Fixed return")

    plt.figure()
    plt.plot(label_for_plot, bankroll_array_rue_worst, 'r-', label='Min')
    plt.plot(label_for_plot, bankroll_array_rue_best, 'g-', label='Max')
    plt.plot(label_for_plot, bankroll_array_rue_avg, 'y-', label='Avg')
    plt.axhline(y=100, color='k', linestyle='-')
    plt.xlabel("Match")
    plt.ylabel("Bankroll")
    plt.legend(loc=0, ncol=2)
    plt.title("Case " + str(case) + " - Variance-adjusted")

    plt.figure()
    plt.plot(label_for_plot, bankroll_array_kelly_worst, 'r-', label='Min')
    plt.plot(label_for_plot, bankroll_array_kelly_best, 'g-', label='Max')
    plt.plot(label_for_plot, bankroll_array_kelly_avg, 'y-', label='Avg')
    plt.axhline(y=100, color='k', linestyle='-')
    plt.xlabel("Match")
    plt.ylabel("Bankroll")
    plt.legend(loc=0, ncol=2)
    plt.title("Case " + str(case) + " - Kelly")
    plt.show()

def betting_sim2(testdata_path, case = 1):

    outer_bankroll_array = []
    for i in range (5):
        nn = Nn(from_file="C:/Users/Mii Erik/OneDrive/Dokumenter/Skole/Masteroppgave/master/nets/betting_sim_case" + str(case) + "_" + str(i) + ".h5")

        inner_bankroll_dict = {}

        for k in [15, 30, 45, 60, 75, 90]:
            matches = []
            test_path = testdata_path
            with open(test_path) as f:
                reader = csv.reader(f)

                for row in reader:
                    input = row[1:-4]
                    if case == 1 or case == 4:
                        input = row[1:-4]
                    elif case == 2 or case == 5:
                        input = row[1:5] + row[-5:-4]
                    elif case == 3 or case == 6:
                        input = row[5:-4]

                    if row[-1] == "home" and int(row[-5]) == k:
                        matches.append([input, [1, 0, 0], row[-4], row[-3], row[0]])
                    elif row[-1] == "none" and int(row[-5]) == k:
                        matches.append([input, [0, 1, 0], row[-4], row[-3], row[0]])
                    elif row[-1] == "away" and int(row[-5]) ==k:
                        matches.append([input, [0, 0, 1], row[-4], row[-3], row[0]])

            sorted_matches = sorted(matches, key=lambda x: datetime.strptime(x[2], "%Y-%m-%d %H:%M:%S"), reverse=False)

            predictions = nn.run_for_betting(sorted_matches)

            odds_set = {}
            id = "000"
            odds_for_id = []

            with open("odds/uf_lo_export_nextgoal.txt", "r") as oddsfile:
                filereader = csv.reader(oddsfile)

                next(filereader)

                for row in filereader:
                    data = row[0].split('\t')
                    if id != data[1]:
                        odds_set[id] = odds_for_id
                        id = data[1]
                        odds_for_id = []
                    odds_for_id.append(data)
                odds_set[id] = odds_for_id

            #Fixed bet
            betting_strategi = "fixed_bet"
            bankroll_array_fixed_bet_inner = []
            total_units_bettet = 0
            bankroll = 100
            top_unit = 0
            corrects = 0
            total_odds = 0
            total_bets_taken = 0
            bankroll_array_fixed_bet_inner.append(bankroll)

            for j, match in enumerate(sorted_matches):
                match_id = match[-1]
                placing_time = datetime.strptime(match[2], "%Y-%m-%d %H:%M:%S")
                deciding_time = datetime.strptime(match[3], "%Y-%m-%d %H:%M:%S")

                prediction = predictions[j]

                odds_for_match = odds_set[match_id]
                odds_data = odds_for_match[0]
                for each_update in odds_for_match:
                    time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
                    if time > placing_time:
                        break
                    odds_data = each_update

                odds = [0.97 / float(odds_data[5]), 0.97 / float(odds_data[7]), 0.97 / float(odds_data[9])]

                index_highest_prob = np.argmax(prediction)
                units_to_bet = 1

                expected_value = odds[index_highest_prob]*prediction[index_highest_prob] - 1

                if expected_value <= 0:
                    bankroll_array_fixed_bet_inner.append(bankroll)
                    continue

                if odds[index_highest_prob] <= 1:
                    bankroll_array_fixed_bet_inner.append(bankroll)
                    continue

                total_units_bettet += units_to_bet
                total_bets_taken += 1
                bankroll -= units_to_bet

                if units_to_bet > top_unit:
                    top_unit = units_to_bet

                if index_highest_prob == 0:
                    if match[1] == [1, 0, 0]:
                        total_odds += odds[index_highest_prob]
                        reward = units_to_bet * odds[index_highest_prob]
                        bankroll += reward
                        corrects += 1

                elif index_highest_prob == 1:
                    if match[1] == [0, 1, 0]:
                        reward = units_to_bet * odds[index_highest_prob]
                        bankroll += reward
                        corrects += 1
                        total_odds += odds[index_highest_prob]

                else:
                    if match[1] == [0, 0, 1]:
                        reward = units_to_bet * odds[index_highest_prob]
                        bankroll += reward
                        corrects += 1
                        total_odds += odds[index_highest_prob]

                bankroll_array_fixed_bet_inner.append(bankroll)


            print("\nYou have this left after the fixed-bet simulation:", bankroll)
            if total_bets_taken == 0:
                print("ROI: 0!!!, no bets were taken")
                inner_bankroll_dict[str(k) + betting_strategi] = [bankroll_array_fixed_bet_inner,
                                                                  0]

            else:
                print("ROI:", ((bankroll - 100) / total_units_bettet) * 100)
                inner_bankroll_dict[str(k) + betting_strategi] = [bankroll_array_fixed_bet_inner,
                                                                  ((bankroll - 100) / total_units_bettet) * 100]

            # Fixed return
            betting_strategi = "fixed_return"
            bankroll_array_fix_return_inner = []
            total_units_bettet = 0
            bankroll = 100
            top_unit = 0
            corrects = 0
            total_odds = 0
            total_bets_taken = 0
            bankroll_array_fix_return_inner.append(bankroll)

            for j, match in enumerate(sorted_matches):
                match_id = match[-1]
                placing_time = datetime.strptime(match[2], "%Y-%m-%d %H:%M:%S")
                deciding_time = datetime.strptime(match[3], "%Y-%m-%d %H:%M:%S")

                prediction = predictions[j]

                odds_for_match = odds_set[match_id]
                odds_data = odds_for_match[0]
                for each_update in odds_for_match:
                    time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
                    if time > placing_time:
                        break
                    odds_data = each_update

                odds = [0.97 / float(odds_data[5]), 0.97 / float(odds_data[7]), 0.97 / float(odds_data[9])]

                index_highest_prob = np.argmax(prediction)
                units_to_bet = 1/odds[index_highest_prob]

                expected_value = odds[index_highest_prob]*prediction[index_highest_prob] - 1

                if odds[index_highest_prob] <= 1:
                    bankroll_array_fix_return_inner.append(bankroll)
                    continue

                if expected_value <= 0:
                    bankroll_array_fix_return_inner.append(bankroll)
                    continue

                total_units_bettet += units_to_bet
                total_bets_taken += 1
                bankroll -= units_to_bet

                if units_to_bet > top_unit:
                    top_unit = units_to_bet

                if index_highest_prob == 0:
                    if match[1] == [1, 0, 0]:
                        total_odds += odds[index_highest_prob]
                        reward = units_to_bet * odds[index_highest_prob]
                        bankroll += reward
                        corrects += 1

                elif index_highest_prob == 1:
                    if match[1] == [0, 1, 0]:
                        reward = units_to_bet * odds[index_highest_prob]
                        bankroll += reward
                        corrects += 1
                        total_odds += odds[index_highest_prob]

                else:
                    if match[1] == [0, 0, 1]:
                        reward = units_to_bet * odds[index_highest_prob]
                        bankroll += reward
                        corrects += 1
                        total_odds += odds[index_highest_prob]

                bankroll_array_fix_return_inner.append(bankroll)

            print("\nYou have this left after the fixed-return simulation:", bankroll)
            if total_units_bettet == 0:
                print("ROI: 0! No bets taken")
                inner_bankroll_dict[str(k) + betting_strategi] = [bankroll_array_fix_return_inner,0]
            else:
                inner_bankroll_dict[str(k) + betting_strategi] = [bankroll_array_fix_return_inner,
                                                                  ((bankroll - 100) / total_units_bettet) * 100]
                print("ROI:", ((bankroll - 100) / total_units_bettet) * 100)


            #Kelly
            betting_strategi = "kelly"
            bankroll_array_kelly_inner = []
            total_units_bettet = 0
            bankroll = 100
            top_unit = 0
            corrects = 0
            total_odds = 0
            total_bets_taken = 0
            bankroll_array_kelly_inner.append(bankroll)

            for j, match in enumerate(sorted_matches):
                match_id = match[-1]
                placing_time = datetime.strptime(match[2], "%Y-%m-%d %H:%M:%S")
                deciding_time = datetime.strptime(match[3], "%Y-%m-%d %H:%M:%S")

                prediction = predictions[j]

                odds_for_match = odds_set[match_id]
                odds_data = odds_for_match[0]
                for each_update in odds_for_match:
                    time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
                    if time > placing_time:
                        break
                    odds_data = each_update

                odds = [0.97 / float(odds_data[5]), 0.97 / float(odds_data[7]), 0.97 / float(odds_data[9])]

                index_highest_prob = np.argmax(prediction)
                fraction=0.25
                units_to_bet = fraction*bankroll*((prediction[index_highest_prob]*odds[index_highest_prob] - 1)/(odds[index_highest_prob] - 1))

                expected_value = odds[index_highest_prob]*prediction[index_highest_prob] - 1

                if odds[index_highest_prob] <= 1:
                    bankroll_array_kelly_inner.append(bankroll)
                    continue

                if expected_value <= 0:
                    bankroll_array_kelly_inner.append(bankroll)
                    continue

                if units_to_bet <= 0:
                    bankroll_array_kelly_inner.append(bankroll)
                    continue
                total_units_bettet += units_to_bet
                total_bets_taken += 1
                bankroll -= units_to_bet

                if units_to_bet > top_unit:
                    top_unit = units_to_bet

                if index_highest_prob == 0:
                    if match[1] == [1, 0, 0]:
                        total_odds += odds[index_highest_prob]
                        reward = units_to_bet * odds[index_highest_prob]
                        bankroll += reward
                        corrects += 1
                elif index_highest_prob == 1:
                    if match[1] == [0, 1, 0]:
                        reward = units_to_bet * odds[index_highest_prob]
                        bankroll += reward
                        corrects += 1
                        total_odds += odds[index_highest_prob]

                else:
                    if match[1] == [0, 0, 1]:
                        reward = units_to_bet * odds[index_highest_prob]
                        bankroll += reward
                        corrects += 1
                        total_odds += odds[index_highest_prob]

                bankroll_array_kelly_inner.append(bankroll)


            print("\nYou have this left after the kelly simulation:", bankroll)
            if total_units_bettet == 0:
                print("ROI: 0!! No bets taken")
                inner_bankroll_dict[str(k) + betting_strategi] = [bankroll_array_kelly_inner,0]
            else:
                print("ROI:", ((bankroll - 100) / total_units_bettet) * 100)
                inner_bankroll_dict[str(k) + betting_strategi] = [bankroll_array_kelly_inner,
                                                      ((bankroll - 100) / total_units_bettet) * 100]


            #Variance-adjusteed
            betting_strategi = "rue_sal"
            bankroll_array_rue_inner = []
            total_units_bettet = 0
            bankroll = 100
            top_unit = 0
            corrects = 0
            total_odds = 0
            total_bets_taken = 0
            bankroll_array_rue_inner.append(bankroll)

            for j, match in enumerate(sorted_matches):
                match_id = match[-1]
                placing_time = datetime.strptime(match[2], "%Y-%m-%d %H:%M:%S")
                deciding_time = datetime.strptime(match[3], "%Y-%m-%d %H:%M:%S")

                prediction = predictions[j]

                odds_for_match = odds_set[match_id]
                odds_data = odds_for_match[0]
                for each_update in odds_for_match:
                    time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
                    if time > placing_time:
                        break
                    odds_data = each_update

                odds = [0.97 / float(odds_data[5]), 0.97 / float(odds_data[7]), 0.97 / float(odds_data[9])]

                index_highest_prob = np.argmax(prediction)
                fraction = 0.25

                units_to_bet = (1)/((2*odds[index_highest_prob])*(1-prediction[index_highest_prob]))

                expected_value = odds[index_highest_prob] * prediction[index_highest_prob] - 1

                if odds[index_highest_prob] <= 1:
                    bankroll_array_rue_inner.append(bankroll)
                    continue

                if expected_value <= 0:
                    bankroll_array_rue_inner.append(bankroll)
                    continue

                if units_to_bet <= 0:
                    bankroll_array_rue_inner.append(bankroll)
                    continue

                total_units_bettet += units_to_bet
                total_bets_taken += 1
                bankroll -= units_to_bet

                if units_to_bet > top_unit:
                    top_unit = units_to_bet

                if index_highest_prob == 0:
                    if match[1] == [1, 0, 0]:
                        total_odds += odds[index_highest_prob]
                        reward = units_to_bet * odds[index_highest_prob]
                        bankroll += reward
                        corrects += 1
                elif index_highest_prob == 1:
                    if match[1] == [0, 1, 0]:
                        reward = units_to_bet * odds[index_highest_prob]
                        bankroll += reward
                        corrects += 1
                        total_odds += odds[index_highest_prob]

                else:
                    if match[1] == [0, 0, 1]:
                        reward = units_to_bet * odds[index_highest_prob]
                        bankroll += reward
                        corrects += 1
                        total_odds += odds[index_highest_prob]

                bankroll_array_rue_inner.append(bankroll)

            print("\nYou have this left after the variance-adjusted simulation:", bankroll)
            if total_units_bettet == 0:
                print("ROI: 0!! No bets taken")
                inner_bankroll_dict[str(k) + betting_strategi] = [bankroll_array_rue_inner,0]
            else:
                inner_bankroll_dict[str(k) + betting_strategi] = [bankroll_array_rue_inner,
                                                                  ((bankroll - 100) / total_units_bettet) * 100]

                print("ROI:", ((bankroll - 100) / total_units_bettet) * 100)


        outer_bankroll_array.append(inner_bankroll_dict)
        print(i)


    avg_dict = {}
    times = ["15", "30", "45", "60", "75", "90"]
    strategies = ["fixed_bet", "fixed_return", "kelly", "rue_sal"]
    avg_roi_dict = {}
    max_roi_dict = {}

    for strategy in strategies:
        for time in times:
            avg_roi_dict[time + strategy] = ((outer_bankroll_array[0][time + strategy][1] +
                                              outer_bankroll_array[1][time + strategy][1] +
                                              outer_bankroll_array[2][time + strategy][1] +
                                              outer_bankroll_array[3][time + strategy][1] +
                                              outer_bankroll_array[4][time + strategy][1]) / 5)
            max_roi_dict[time + strategy] = max(outer_bankroll_array[0][time + strategy][1],
                                              outer_bankroll_array[1][time + strategy][1],
                                              outer_bankroll_array[2][time + strategy][1],
                                              outer_bankroll_array[3][time + strategy][1],
                                              outer_bankroll_array[4][time + strategy][1])


    for strategy in strategies:
        for time in times:
            print (time, "-",strategy, ": Avg:", avg_roi_dict[time + strategy], ". Max:", max_roi_dict[time + strategy])

def betting_sim3(testdata_path, case = 1):
    bankroll_array_kelly1 = []
    bankroll_array_variance1 = []
    bankroll_array_kelly2 = []
    bankroll_array_variance2 = []
    for i in range (5):

        nn = Nn(
            from_file="C:/Users/Mii Erik/OneDrive/Dokumenter/Skole/Masteroppgave/master/nets/betting_sim_case" + str(case) + "_" + str(
                i) + ".h5")

        matches = []
        test_path = testdata_path
        with open(test_path) as f:
            reader = csv.reader(f)

            for row in reader:
                input = row[1:-4]
                if case == 1 or case == 4:
                    input = row[1:-4]
                elif case == 2 or case == 5:
                    input = row[1:5] + row[-5:-4]
                elif case == 3 or case == 6:
                    input = row[5:-4]

                if row[-1] == "home":
                    matches.append([input, [1, 0, 0], row[-4], row[-3], row[-5], row[0]])
                elif row[-1] == "none":
                    matches.append([input, [0, 1, 0], row[-4], row[-3], row[-5], row[0]])
                elif row[-1] == "away":
                    matches.append([input, [0, 0, 1], row[-4], row[-3], row[-5], row[0]])

        sorted_matches = sorted(matches, key=lambda x: datetime.strptime(x[2], "%Y-%m-%d %H:%M:%S"), reverse=False)

        predictions = nn.run_for_betting(sorted_matches)

        match_and_predictions = {}

        for l, each in enumerate(sorted_matches):
            if each[-1] in match_and_predictions:
                match_and_predictions[each[-1]].append((each, predictions[l]))
            else:
                match_and_predictions[each[-1]] = [(each, predictions[l])]

        odds_set = {}
        id = "000"
        odds_for_id = []

        with open("odds/uf_lo_export_nextgoal.txt", "r") as oddsfile:
            filereader = csv.reader(oddsfile)

            next(filereader)

            for row in filereader:
                data = row[0].split('\t')
                if id != data[1]:
                    odds_set[id] = odds_for_id
                    id = data[1]
                    odds_for_id = []
                odds_for_id.append(data)
            odds_set[id] = odds_for_id

        betting_strategi = "kelly1"
        bankroll_array_kelly1_inner = []
        total_units_bettet = 0
        bankroll = 100
        top_unit = 0
        corrects = 0
        total_odds = 0
        total_bets_taken = 0
        bankroll_array_kelly1_inner.append(bankroll)

        for j, match_id in enumerate(match_and_predictions):
            active_bets = []

            odds_for_match = odds_set[match_id]
            for each in match_and_predictions[match_id]:

                match = each[0]
                prediction = each[1]
                placing_time = datetime.strptime(match[2], "%Y-%m-%d %H:%M:%S")
                deciding_time = datetime.strptime(match[3], "%Y-%m-%d %H:%M:%S")

                for bet in active_bets[:]:
                    if bet[2] < placing_time:
                        if bet[1] == True:
                            bankroll += bet[0]
                        active_bets.remove(bet)
                odds_data = odds_for_match[0]
                for each_update in odds_for_match:
                    time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
                    if time > placing_time:
                        break
                    odds_data = each_update

                odds = [0.97 / float(odds_data[5]), 0.97 / float(odds_data[7]), 0.97 / float(odds_data[9])]

                index_highest_prob = np.argmax(prediction)
                fraction=0.25
                units_to_bet = fraction*bankroll*((prediction[index_highest_prob]*odds[index_highest_prob] - 1)/(odds[index_highest_prob] - 1))

                expected_value = odds[index_highest_prob]*prediction[index_highest_prob] - 1

                if odds[index_highest_prob] <= 1 or expected_value <= 0 or units_to_bet <= 0:
                    if match[-2] == "90":
                        for bet in active_bets[:]:
                            if bet[1] == True:
                                bankroll += bet[0]
                        bankroll_array_kelly1_inner.append(bankroll)

                    continue

                if active_bets:
                    can_bet = False
                    total_betted_on_same = 0
                    extra_units_to_bet = units_to_bet
                    for each_bet in active_bets:
                        if each_bet[-1] == index_highest_prob:
                            total_betted_on_same += each_bet[3]

                    if total_betted_on_same != 0:
                        difference = units_to_bet - total_betted_on_same
                        percentage_difference = difference/total_betted_on_same
                        if percentage_difference > 0:
                            can_bet = True
                            extra_units_to_bet = difference
                    else:
                        can_bet = True

                    if not can_bet:
                        if match[-2] == "90":
                            for bet in active_bets[:]:
                                if bet[1] == True:
                                    bankroll += bet[0]
                            bankroll_array_kelly1_inner.append(bankroll)
                        continue
                    else:
                        units_to_bet = extra_units_to_bet

                if units_to_bet > bankroll:
                    if match[-2] == "90":
                        for bet in active_bets[:]:
                            if bet[1] == True:
                                bankroll += bet[0]
                        bankroll_array_kelly1_inner.append(bankroll)
                    continue

                if units_to_bet > 0:
                    total_units_bettet += units_to_bet
                    total_bets_taken += 1
                    bankroll -= units_to_bet
                    if units_to_bet > top_unit:
                        top_unit = units_to_bet

                    if index_highest_prob == 0:
                        reward = units_to_bet * odds[index_highest_prob]
                        if match[1] == [1, 0, 0]:
                            total_odds += odds[index_highest_prob]
                            corrects += 1
                            active_bets.append([reward, True, deciding_time, units_to_bet, 0])
                        else:
                            active_bets.append([reward, False, deciding_time, units_to_bet, 0])
                    elif index_highest_prob == 1:
                        reward = units_to_bet * odds[index_highest_prob]
                        if match[1] == [0, 1, 0]:
                            corrects += 1
                            total_odds += odds[index_highest_prob]
                            active_bets.append([reward, True, deciding_time, units_to_bet, 1])
                        else:
                            active_bets.append([reward, False, deciding_time, units_to_bet, 1])

                    else:
                        reward = units_to_bet * odds[index_highest_prob]
                        if match[1] == [0, 0, 1]:
                            corrects += 1
                            total_odds += odds[index_highest_prob]
                            active_bets.append([reward, True, deciding_time, units_to_bet, 2])
                        else:
                            active_bets.append([reward, False, deciding_time, units_to_bet, 2])

                if match[-2] == "90":
                    for bet in active_bets[:]:
                        if bet[1] == True:
                            bankroll += bet[0]
                    bankroll_array_kelly1_inner.append(bankroll)


        print("\nYou have this left after the kelly simulation:", bankroll)
        print("ROI:", ((bankroll - 100) / total_units_bettet) * 100)
        bankroll_array_kelly1.append(
            [bankroll_array_kelly1_inner, ((bankroll - 100) / total_units_bettet) * 100])

        betting_strategi = "rue_sal1"
        bankroll_array_variance1_inner = []
        total_units_bettet = 0
        bankroll = 100
        top_unit = 0
        corrects = 0
        total_odds = 0
        total_bets_taken = 0
        bankroll_array_variance1_inner.append(bankroll)

        for j, match_id in enumerate(match_and_predictions):
            active_bets = []

            odds_for_match = odds_set[match_id]
            for each in match_and_predictions[match_id]:
                match = each[0]
                prediction = each[1]
                placing_time = datetime.strptime(match[2], "%Y-%m-%d %H:%M:%S")
                deciding_time = datetime.strptime(match[3], "%Y-%m-%d %H:%M:%S")

                for bet in active_bets[:]:
                    if bet[2] < placing_time:
                        if bet[1] == True:
                            bankroll += bet[0]
                        active_bets.remove(bet)
                odds_data = odds_for_match[0]
                for each_update in odds_for_match:
                    time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
                    if time > placing_time:
                        break
                    odds_data = each_update

                odds = [0.97 / float(odds_data[5]), 0.97 / float(odds_data[7]), 0.97 / float(odds_data[9])]

                index_highest_prob = np.argmax(prediction)
                units_to_bet = (1) / ((2 * odds[index_highest_prob]) * (1 - prediction[index_highest_prob]))
                expected_value = odds[index_highest_prob]*prediction[index_highest_prob] - 1

                if odds[index_highest_prob] <= 1 or expected_value <= 0 or units_to_bet <= 0:
                    if match[-2] == "90":
                        for bet in active_bets[:]:
                            if bet[1] == True:
                                bankroll += bet[0]
                        bankroll_array_variance1_inner.append(bankroll)

                    continue

                if active_bets:
                    can_bet = False
                    total_betted_on_same = 0
                    extra_units_to_bet = units_to_bet
                    for each_bet in active_bets:
                        if each_bet[-1] == index_highest_prob:
                            total_betted_on_same += each_bet[3]

                    if total_betted_on_same != 0:
                        difference = units_to_bet - total_betted_on_same
                        percentage_difference = difference/total_betted_on_same
                        if percentage_difference > 0:
                            can_bet = True
                            extra_units_to_bet = difference
                    else:
                        can_bet = True

                    if not can_bet:
                        if match[-2] == "90":
                            for bet in active_bets[:]:
                                if bet[1] == True:
                                    bankroll += bet[0]
                            bankroll_array_variance1_inner.append(bankroll)
                        continue
                    else:
                        units_to_bet = extra_units_to_bet

                if units_to_bet > bankroll:
                    if match[-2] == "90":
                        for bet in active_bets[:]:
                            if bet[1] == True:
                                bankroll += bet[0]
                        bankroll_array_variance1_inner.append(bankroll)
                    continue

                if units_to_bet > 0:

                    total_units_bettet += units_to_bet
                    total_bets_taken += 1
                    bankroll -= units_to_bet
                    if units_to_bet > top_unit:
                        top_unit = units_to_bet

                    if index_highest_prob == 0:
                        reward = units_to_bet * odds[index_highest_prob]
                        if match[1] == [1, 0, 0]:
                            total_odds += odds[index_highest_prob]
                            corrects += 1
                            active_bets.append([reward, True, deciding_time, units_to_bet, 0])
                        else:
                            active_bets.append([reward, False, deciding_time, units_to_bet, 0])
                    elif index_highest_prob == 1:
                        reward = units_to_bet * odds[index_highest_prob]
                        if match[1] == [0, 1, 0]:
                            corrects += 1
                            total_odds += odds[index_highest_prob]
                            active_bets.append([reward, True, deciding_time, units_to_bet, 1])
                        else:
                            active_bets.append([reward, False, deciding_time, units_to_bet, 1])

                    else:
                        reward = units_to_bet * odds[index_highest_prob]
                        if match[1] == [0, 0, 1]:
                            corrects += 1
                            total_odds += odds[index_highest_prob]
                            active_bets.append([reward, True, deciding_time, units_to_bet, 2])
                        else:
                            active_bets.append([reward, False, deciding_time, units_to_bet, 2])

                if match[-2] == "90":
                    for bet in active_bets[:]:
                        if bet[1] == True:
                            bankroll += bet[0]
                    bankroll_array_variance1_inner.append(bankroll)


        print("\nYou have this left after the variance simulation:", bankroll)
        print("ROI:", ((bankroll - 100) / total_units_bettet) * 100)
        bankroll_array_variance1.append(
            [bankroll_array_variance1_inner, ((bankroll - 100) / total_units_bettet) * 100])

        betting_strategi = "kelly2"
        bankroll_array_kelly2_inner = []
        total_units_bettet = 0
        bankroll = 100
        top_unit = 0
        corrects = 0
        total_odds = 0
        total_bets_taken = 0
        bankroll_array_kelly2_inner.append(bankroll)

        for j, match_id in enumerate(match_and_predictions):
            active_bets = []

            odds_for_match = odds_set[match_id]
            for each in match_and_predictions[match_id]:
                match = each[0]
                prediction = each[1]
                placing_time = datetime.strptime(match[2], "%Y-%m-%d %H:%M:%S")
                deciding_time = datetime.strptime(match[3], "%Y-%m-%d %H:%M:%S")

                for bet in active_bets[:]:
                    if bet[2] < placing_time:
                        if bet[1] == True:
                            bankroll += bet[0]
                        active_bets.remove(bet)
                odds_data = odds_for_match[0]
                for each_update in odds_for_match:
                    time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
                    if time > placing_time:
                        break
                    odds_data = each_update

                odds = [0.97 / float(odds_data[5]), 0.97 / float(odds_data[7]), 0.97 / float(odds_data[9])]

                index_highest_prob = np.argmax(prediction)
                fraction=0.25
                units_to_bet = fraction*bankroll*((prediction[index_highest_prob]*odds[index_highest_prob] - 1)/(odds[index_highest_prob] - 1))

                expected_value = odds[index_highest_prob]*prediction[index_highest_prob] - 1

                if odds[index_highest_prob] <= 1 or expected_value <= 0 or units_to_bet <= 0:
                    if match[-2] == "90":
                        for bet in active_bets[:]:
                            if bet[1] == True:
                                bankroll += bet[0]
                        bankroll_array_kelly2_inner.append(bankroll)

                    continue

                if active_bets:
                    total_betted_on_same = 0
                    total_betted_on_different = 0
                    index_of_last_bet = 0

                    for each_bet in active_bets:
                        if each_bet[-1] == index_highest_prob:
                            total_betted_on_same += each_bet[3]
                        else:
                            total_betted_on_different += each_bet[3]

                        index_of_last_bet = each_bet[-1]

                    bet_to_nullify_other_bets = 0
                    if index_of_last_bet != index_highest_prob:
                        bet_to_nullify_other_bets = total_betted_on_different / odds[index_highest_prob]

                    kellys_x = 0
                    if total_betted_on_same != 0:
                        difference = units_to_bet - total_betted_on_same
                        if difference > 0:
                            kellys_x = difference

                    else:
                        kellys_x = units_to_bet

                    units_to_bet = kellys_x + bet_to_nullify_other_bets


                if units_to_bet > bankroll:
                    if match[-2] == "90":
                        for bet in active_bets[:]:
                            if bet[1] == True:
                                bankroll += bet[0]
                        bankroll_array_kelly2_inner.append(bankroll)
                    continue

                if units_to_bet > 0:

                    total_units_bettet += units_to_bet
                    total_bets_taken += 1
                    bankroll -= units_to_bet
                    if units_to_bet > top_unit:
                        top_unit = units_to_bet

                    if index_highest_prob == 0:
                        reward = units_to_bet * odds[index_highest_prob]
                        if match[1] == [1, 0, 0]:
                            total_odds += odds[index_highest_prob]
                            corrects += 1
                            active_bets.append([reward, True, deciding_time, units_to_bet, 0])
                        else:
                            active_bets.append([reward, False, deciding_time, units_to_bet, 0])
                    elif index_highest_prob == 1:
                        reward = units_to_bet * odds[index_highest_prob]
                        if match[1] == [0, 1, 0]:
                            corrects += 1
                            total_odds += odds[index_highest_prob]
                            active_bets.append([reward, True, deciding_time, units_to_bet, 1])
                        else:
                            active_bets.append([reward, False, deciding_time, units_to_bet, 1])

                    else:
                        reward = units_to_bet * odds[index_highest_prob]
                        if match[1] == [0, 0, 1]:
                            corrects += 1
                            total_odds += odds[index_highest_prob]
                            active_bets.append([reward, True, deciding_time, units_to_bet, 2])
                        else:
                            active_bets.append([reward, False, deciding_time, units_to_bet, 2])

                if match[-2] == "90":
                    for bet in active_bets[:]:
                        if bet[1] == True:
                            bankroll += bet[0]
                    bankroll_array_kelly2_inner.append(bankroll)


        print("\nYou have this left after the kelly simulation:", bankroll)
        print("ROI:", ((bankroll - 100) / total_units_bettet) * 100)
        bankroll_array_kelly2.append(
            [bankroll_array_kelly2_inner, ((bankroll - 100) / total_units_bettet) * 100])

        betting_strategi = "rue_sal2"
        bankroll_array_variance2_inner = []
        total_units_bettet = 0
        bankroll = 100
        top_unit = 0
        corrects = 0
        total_odds = 0
        total_bets_taken = 0
        bankroll_array_variance2_inner.append(bankroll)

        for j, match_id in enumerate(match_and_predictions):
            active_bets = []
            odds_for_match = odds_set[match_id]
            for each in match_and_predictions[match_id]:
                match = each[0]
                prediction = each[1]
                placing_time = datetime.strptime(match[2], "%Y-%m-%d %H:%M:%S")
                deciding_time = datetime.strptime(match[3], "%Y-%m-%d %H:%M:%S")

                for bet in active_bets[:]:
                    if bet[2] < placing_time:
                        if bet[1] == True:
                            bankroll += bet[0]
                        active_bets.remove(bet)

                odds_data = odds_for_match[0]
                for each_update in odds_for_match:
                    time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
                    if time > placing_time:
                        break
                    odds_data = each_update

                odds = [0.97 / float(odds_data[5]), 0.97 / float(odds_data[7]), 0.97 / float(odds_data[9])]

                index_highest_prob = np.argmax(prediction)
                units_to_bet = (1) / ((2 * odds[index_highest_prob]) * (1 - prediction[index_highest_prob]))
                expected_value = odds[index_highest_prob]*prediction[index_highest_prob] - 1

                if odds[index_highest_prob] <= 1 or expected_value <= 0 or units_to_bet <= 0:
                    if match[-2] == "90":
                        for bet in active_bets[:]:
                            if bet[1] == True:
                                bankroll += bet[0]
                        bankroll_array_variance2_inner.append(bankroll)
                    continue

                if active_bets:
                    total_betted_on_same = 0
                    total_betted_on_different = 0
                    index_of_last_bet = 0

                    for each_bet in active_bets:
                        if each_bet[-1] == index_highest_prob:
                            total_betted_on_same += each_bet[3]
                        else:
                            total_betted_on_different += each_bet[3]

                        index_of_last_bet = each_bet[-1]

                    bet_to_nullify_other_bets = 0
                    if index_of_last_bet != index_highest_prob:
                        bet_to_nullify_other_bets = total_betted_on_different / odds[index_highest_prob]

                    var_x = 0
                    if total_betted_on_same != 0:
                        difference = units_to_bet - total_betted_on_same
                        if difference > 0:
                            var_x = difference

                    else:
                        var_x = units_to_bet

                    units_to_bet = var_x + bet_to_nullify_other_bets

                if units_to_bet > bankroll:
                    if match[-2] == "90":
                        for bet in active_bets[:]:
                            if bet[1] == True:
                                bankroll += bet[0]
                        bankroll_array_variance2_inner.append(bankroll)
                    continue

                if units_to_bet > 0:
                    total_units_bettet += units_to_bet
                    total_bets_taken += 1
                    bankroll -= units_to_bet
                    if units_to_bet > top_unit:
                        top_unit = units_to_bet

                    if index_highest_prob == 0:
                        reward = units_to_bet * odds[index_highest_prob]
                        if match[1] == [1, 0, 0]:
                            total_odds += odds[index_highest_prob]
                            corrects += 1
                            active_bets.append([reward, True, deciding_time, units_to_bet, 0])
                        else:
                            active_bets.append([reward, False, deciding_time, units_to_bet, 0])
                    elif index_highest_prob == 1:
                        reward = units_to_bet * odds[index_highest_prob]
                        if match[1] == [0, 1, 0]:
                            corrects += 1
                            total_odds += odds[index_highest_prob]
                            active_bets.append([reward, True, deciding_time, units_to_bet, 1])
                        else:
                            active_bets.append([reward, False, deciding_time, units_to_bet, 1])

                    else:
                        reward = units_to_bet * odds[index_highest_prob]
                        if match[1] == [0, 0, 1]:
                            corrects += 1
                            total_odds += odds[index_highest_prob]
                            active_bets.append([reward, True, deciding_time, units_to_bet, 2])
                        else:
                            active_bets.append([reward, False, deciding_time, units_to_bet, 2])

                if match[-2] == "90":
                    for bet in active_bets[:]:
                        if bet[1] == True:
                            bankroll += bet[0]
                    bankroll_array_variance2_inner.append(bankroll)


        print("\nYou have this left after the variance simulation:", bankroll)
        print("ROI:", ((bankroll - 100) / total_units_bettet) * 100)
        bankroll_array_variance2.append(
            [bankroll_array_variance2_inner, ((bankroll - 100) / total_units_bettet) * 100])

    bankroll_array_kelly1_avg = []
    bankroll_array_variance1_avg = []
    bankroll_array_kelly2_avg = []
    bankroll_array_variance2_avg = []

    bankroll_array_kelly1_best = bankroll_array_kelly1[0][0]
    bankroll_array_kelly1_best_roi = bankroll_array_kelly1[0][1]
    bankroll_array_variance1_best = bankroll_array_variance1[0][0]
    bankroll_array_variance1_best_roi = bankroll_array_variance1[0][1]
    bankroll_array_kelly2_best = bankroll_array_kelly2[0][0]
    bankroll_array_kelly2_best_roi = bankroll_array_kelly2[0][1]
    bankroll_array_variance2_best = bankroll_array_variance2[0][0]
    bankroll_array_variance2_best_roi = bankroll_array_variance2[0][1]

    bankroll_array_kelly1_worst = bankroll_array_kelly1[0][0]
    bankroll_array_kelly1_worst_roi = bankroll_array_kelly1[0][1]
    bankroll_array_variance1_worst = bankroll_array_variance1[0][0]
    bankroll_array_variance1_worst_roi = bankroll_array_variance1[0][1]
    bankroll_array_kelly2_worst = bankroll_array_kelly2[0][0]
    bankroll_array_kelly2_worst_roi = bankroll_array_kelly2[0][1]
    bankroll_array_variance2_worst = bankroll_array_variance2[0][0]
    bankroll_array_variance2_worst_roi = bankroll_array_variance2[0][1]


    for j, _ in enumerate(bankroll_array_kelly1[0][0]):
        bankroll_array_kelly1_avg.append((bankroll_array_kelly1[0][0][j] + bankroll_array_kelly1[1][0][j]
                                         + bankroll_array_kelly1[2][0][j] + bankroll_array_kelly1[3][0][j]
                                         + bankroll_array_kelly1[4][0][j]) / 5)

        bankroll_array_variance1_avg.append((bankroll_array_variance1[0][0][j] + bankroll_array_variance1[1][0][j]
                                         + bankroll_array_variance1[2][0][j] + bankroll_array_variance1[3][0][j]
                                         + bankroll_array_variance1[4][0][j]) / 5)

        bankroll_array_kelly2_avg.append((bankroll_array_kelly2[0][0][j] + bankroll_array_kelly2[1][0][j]
                                          + bankroll_array_kelly2[2][0][j] + bankroll_array_kelly2[3][0][j]
                                          + bankroll_array_kelly2[4][0][j]) / 5)

        bankroll_array_variance2_avg.append((bankroll_array_variance2[0][0][j] + bankroll_array_variance2[1][0][j]
                                             + bankroll_array_variance2[2][0][j] + bankroll_array_variance2[3][0][j]
                                             + bankroll_array_variance2[4][0][j]) / 5)

    avg_roi_kelly1 = 0
    avg_roi_variance1 = 0
    avg_roi_kelly2 = 0
    avg_roi_variance2 = 0

    for i in range(5):
        avg_roi_kelly1 += bankroll_array_kelly1[i][1]
        avg_roi_variance1 += bankroll_array_variance1[i][1]
        avg_roi_kelly2 += bankroll_array_kelly2[i][1]
        avg_roi_variance2 += bankroll_array_variance2[i][1]

        if bankroll_array_kelly1[i][1] > bankroll_array_kelly1_best_roi:
            bankroll_array_kelly1_best = bankroll_array_kelly1[i][0]
            bankroll_array_kelly1_best_roi = bankroll_array_kelly1[i][1]

        if bankroll_array_variance1[i][1] > bankroll_array_variance1_best_roi:
            bankroll_array_variance1_best = bankroll_array_variance1[i][0]
            bankroll_array_variance1_best_roi = bankroll_array_variance1[i][1]

        if bankroll_array_kelly2[i][1] > bankroll_array_kelly2_best_roi:
            bankroll_array_kelly2_best = bankroll_array_kelly2[i][0]
            bankroll_array_kelly2_best_roi = bankroll_array_kelly2[i][1]

        if bankroll_array_variance2[i][1] > bankroll_array_variance2_best_roi:
            bankroll_array_variance2_best = bankroll_array_variance2[i][0]
            bankroll_array_variance2_best_roi = bankroll_array_variance2[i][1]

        if bankroll_array_kelly1[i][1] < bankroll_array_kelly1_worst_roi:
            bankroll_array_kelly1_worst = bankroll_array_kelly1[i][0]
            bankroll_array_kelly1_worst_roi = bankroll_array_kelly1[i][1]

        if bankroll_array_variance1[i][1] < bankroll_array_variance1_worst_roi:
            bankroll_array_variance1_worst = bankroll_array_variance1[i][0]
            bankroll_array_variance1_worst_roi = bankroll_array_variance1[i][1]

        if bankroll_array_kelly2[i][1] < bankroll_array_kelly2_worst_roi:
            bankroll_array_kelly2_worst = bankroll_array_kelly2[i][0]
            bankroll_array_kelly2_worst_roi = bankroll_array_kelly2[i][1]

        if bankroll_array_variance2[i][1] < bankroll_array_variance2_worst_roi:
            bankroll_array_variance2_worst = bankroll_array_variance2[i][0]
            bankroll_array_variance2_worst_roi = bankroll_array_variance2[i][1]


    avg_roi_kelly1 /= 5
    avg_roi_variance1 /= 5
    avg_roi_kelly2 /= 5
    avg_roi_variance2 /= 5

    print("\n")
    print("Kelly avg:", avg_roi_kelly1, ". Max:", bankroll_array_kelly1_best_roi, ". Min:", bankroll_array_kelly1_worst_roi)
    print("Variance avg:", avg_roi_variance1, ". Max:", bankroll_array_variance1_best_roi, ". Min:", bankroll_array_variance1_worst_roi)
    print("Kelly2 avg:", avg_roi_kelly2, ". Max:", bankroll_array_kelly2_best_roi, ". Min:", bankroll_array_kelly2_worst_roi)
    print("Variance2 avg:", avg_roi_variance2, ". Max:", bankroll_array_variance2_best_roi, ". Min:", bankroll_array_variance2_worst_roi)

    label_for_plot = []
    for j, _ in enumerate(bankroll_array_kelly1[0][0]):
        label_for_plot.append(j)

    plt.figure()
    plt.plot(label_for_plot, bankroll_array_kelly1_worst, 'r-', label='Min')
    plt.plot(label_for_plot, bankroll_array_kelly1_best, 'g-', label='Max')
    plt.plot(label_for_plot, bankroll_array_kelly1_avg, 'y-', label='Avg')
    plt.axhline(y=100, color='k', linestyle='-')
    plt.xlabel("Match")
    plt.ylabel("Bankroll")
    plt.legend(loc=0, ncol=2)
    plt.title("Case " + str(case) + " - Continuous Kelly 1")
    print("\nKelly avg:", bankroll_array_kelly1_avg)
    print("Kelly best:", bankroll_array_kelly1_best)

    plt.figure()
    plt.plot(label_for_plot, bankroll_array_variance1_worst, 'r-', label='Min')
    plt.plot(label_for_plot, bankroll_array_variance1_best, 'g-', label='Max')
    plt.plot(label_for_plot, bankroll_array_variance1_avg, 'y-', label='Avg')
    plt.axhline(y=100, color='k', linestyle='-')
    plt.xlabel("Match")
    plt.ylabel("Bankroll")
    plt.legend(loc=0, ncol=2)
    plt.title("Case " + str(case) + " - Continuous variance-adjusted 1")
    print("\nVariance avg:", bankroll_array_variance1_avg)
    print("Variance best:", bankroll_array_variance1_best)
    plt.show()

    plt.figure()
    plt.plot(label_for_plot, bankroll_array_kelly2_worst, 'r-', label='Min')
    plt.plot(label_for_plot, bankroll_array_kelly2_best, 'g-', label='Max')
    plt.plot(label_for_plot, bankroll_array_kelly2_avg, 'y-', label='Avg')
    plt.axhline(y=100, color='k', linestyle='-')
    plt.xlabel("Match")
    plt.ylabel("Bankroll")
    plt.legend(loc=0, ncol=2)
    plt.title("Case " + str(case) + " - Continuous Kelly 2")
    print("\nKelly avg:", bankroll_array_kelly2_avg)
    print("Kelly best:", bankroll_array_kelly2_best)

    plt.figure()
    plt.plot(label_for_plot, bankroll_array_variance2_worst, 'r-', label='Min')
    plt.plot(label_for_plot, bankroll_array_variance2_best, 'g-', label='Max')
    plt.plot(label_for_plot, bankroll_array_variance2_avg, 'y-', label='Avg')
    plt.axhline(y=100, color='k', linestyle='-')
    plt.xlabel("Match")
    plt.ylabel("Bankroll")
    plt.legend(loc=0, ncol=2)
    plt.title("Case " + str(case) + " - Continuous variance-adjusted 2")
    print("\nVariance avg:", bankroll_array_variance2_avg)
    print("Variance best:", bankroll_array_variance2_best)
    plt.show()


case = 5
train_path = "data/training_cases4_6.csv"
test_path_sim1 = "data/test1_cases4_6.csv"
test_path_sim2 = "data/test2_cases4_6.csv"
test_path_sim3 = "data/test3_cases4_6.csv"


#find_best_network_params(training_path = train_path, case = case)

#betting_sim_1(training_path = train_path, testdata_path = test_path_sim1,
               #hidden_layer = [57], hidden_activation="relu", l_rate= 0.002, batch_size=256, case = case)

#betting_sim2(testdata_path = test_path_sim2, case = case)

betting_sim3(testdata_path= test_path_sim3, case = case)