from network_ff import Nn
import csv
import numpy as np
from matplotlib import pyplot as plt
from datetime import datetime
from numpy import array
import itertools


def plot_conf_matrix(matrix, class_names=["Home", "None", "Away"], title="Confusion matrix", figure_nr=3):
    accuracy = np.trace(matrix) / float(np.sum(matrix))
    misclass = 1 - accuracy

    matrix = matrix.astype('float') / matrix.sum(axis=1)[:, np.newaxis]

    cmap = plt.get_cmap("Blues")
    plt.figure(figure_nr)
    plt.imshow(matrix, interpolation="nearest", cmap=cmap)
    plt.title(title)

    tick_marks = np.arange(len(class_names))
    plt.xticks(tick_marks, class_names, rotation=45)
    plt.yticks(tick_marks, class_names)

    thresh = matrix.max() / 2
    fmt = '.2f'

    for i, j in itertools.product(range(matrix.shape[0]), range(matrix.shape[1])):
        plt.text(j, i, format(matrix[i, j], fmt),
                 horizontalalignment="center",
                 color="white" if matrix[i, j] > thresh else "black")

    plt.tight_layout()
    plt.ylabel('Actual outcome')
    plt.xlabel('Predictions')
    print("\naccuracy={:0.4f}; misclass={:0.4f}".format(accuracy, misclass))
def run_acc_test(case, testdata_path):
    returned_for_plots = []
    odds_for_plots = []

    predictions_0 = []
    predictions_10 = []
    predictions_20 = []
    predictions_30 = []
    predictions_40 = []
    predictions_50 = []
    predictions_60 = []
    predictions_70 = []
    predictions_80 = []
    predictions_90 = []
    predictions_100 = []

    odds_0 = []
    odds_10 = []
    odds_20 = []
    odds_30 = []
    odds_40 = []
    odds_50 = []
    odds_60 = []
    odds_70 = []
    odds_80 = []
    odds_90 = []
    odds_100 = []

    labels_for_matrix = []
    predictions_for_matrix = []
    odds_for_matrix = []


    odds_set = {}
    id = "000"
    odds_for_id = []

    with open("odds/uf_lo_export_nextgoal.txt", "r") as oddsfile:
        filereader = csv.reader(oddsfile)

        next(filereader)

        for row in filereader:
            data = row[0].split('\t')
            if id != data[1]:
                odds_set[id] = odds_for_id
                id = data[1]
                odds_for_id = []
            odds_for_id.append(data)
        odds_set[id] = odds_for_id

    for i in range (5):
        nn = Nn(from_file="C:/Users/Mii Erik/OneDrive/Dokumenter/Skole/Masteroppgave/master/nets/betting_sim_case" + str(
            case) + "_" + str(i) + ".h5")

        matches = []
        test_path = testdata_path
        with open(test_path) as f:
            reader = csv.reader(f)

            for row in reader:
                input = row[1:-4]
                if case == 1 or case == 4:
                    input = row[1:-4]
                elif case == 2 or case == 5:
                    input = row[1:5] + row[-5:-4]
                elif case == 3 or case == 6:
                    input = row[5:-4]

                if row[-1] == "home":
                    matches.append([input, [1, 0, 0], row[-4], row[-3], row[0]])
                elif row[-1] == "none":
                    matches.append([input, [0, 1, 0], row[-4], row[-3], row[0]])
                elif row[-1] == "away":
                    matches.append([input, [0, 0, 1], row[-4], row[-3], row[0]])

        sorted_matches = sorted(matches, key=lambda x: datetime.strptime(x[2], "%Y-%m-%d %H:%M:%S"), reverse=False)

        predictions = nn.run_for_betting(sorted_matches)

        for j, match in enumerate(sorted_matches):
            match_id = match[-1]
            placing_time = datetime.strptime(match[2], "%Y-%m-%d %H:%M:%S")
            deciding_time = datetime.strptime(match[3], "%Y-%m-%d %H:%M:%S")

            prediction = predictions[j]

            labels_for_matrix.append(match[1])
            predictions_for_matrix.append(prediction)

            odds_for_match = odds_set[match_id]
            odds_data = odds_for_match[0]
            for each_update in odds_for_match:
                time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
                if time > placing_time:
                    break
                odds_data = each_update

            if match[1] == [1, 0, 0]:
                answer = 0
            elif match[1] == [0, 1, 0]:
                answer = 1
            else:
                answer = 2

            for k, pre in enumerate(prediction):
                true_false = False
                if k == answer:
                    true_false = True
                if pre < 0.05:
                    predictions_0.append(true_false)
                elif pre < 0.15:
                    predictions_10.append(true_false)
                elif pre < 0.25:
                    predictions_20.append(true_false)
                elif pre < 0.35:
                    predictions_30.append(true_false)
                elif pre < 0.45:
                    predictions_40.append(true_false)
                elif pre < 0.55:
                    predictions_50.append(true_false)
                elif pre < 0.65:
                    predictions_60.append(true_false)
                elif pre < 0.75:
                    predictions_70.append(true_false)
                elif pre < 0.85:
                    predictions_80.append(true_false)
                elif pre < 0.95:
                    predictions_90.append(true_false)
                else:
                    predictions_100.append(true_false)

            odds = [float(odds_data[5]), float(odds_data[7]), float(odds_data[9])]
            odds_for_matrix.append(odds)

            for l, odd in enumerate(odds):
                false_true = False
                if l == answer:
                    false_true = True
                if odd < 0.05:
                    odds_0.append(false_true)
                elif odd < 0.15:
                    odds_10.append(false_true)
                elif odd < 0.25:
                    odds_20.append(false_true)
                elif odd < 0.35:
                    odds_30.append(false_true)
                elif odd < 0.45:
                    odds_40.append(false_true)
                elif odd < 0.55:
                    odds_50.append(false_true)
                elif odd < 0.65:
                    odds_60.append(false_true)
                elif odd < 0.75:
                    odds_70.append(false_true)
                elif odd < 0.85:
                    odds_80.append(false_true)
                elif odd < 0.95:
                    odds_90.append(false_true)
                else:
                    odds_100.append(false_true)

    if len(predictions_0)>0:
        trues = 0
        for each in predictions_0:
            if each == True:
                trues+=1

        print("At 0%:", trues/len(predictions_0), "-",trues,"out of",len(predictions_0))
        returned_for_plots.append([0, trues/len(predictions_0)])
    else:
        print("No predictions at 0%")

    if len(predictions_10) > 0:
        trues = 0
        for each in predictions_10:
            if each == True:
                trues += 1

        print("At 10%:", trues/len(predictions_10), "-",trues,"out of",len(predictions_10))
        returned_for_plots.append([10, trues/len(predictions_10)])
    else:
        print("No predictions at 10%")

    if len(predictions_20) > 0:
        trues = 0
        for each in predictions_20:
            if each == True:
                trues += 1

        print("At 20%:", trues/len(predictions_20), "-",trues,"out of",len(predictions_20))
        returned_for_plots.append([20, trues/len(predictions_20)])
    else:
        print("No predictions at 20%")

    if len(predictions_30) > 0:
        trues = 0
        for each in predictions_30:
            if each == True:
                trues += 1

        print("At 30%:", trues/len(predictions_30), "-",trues,"out of",len(predictions_30))
        returned_for_plots.append([30, trues/len(predictions_30)])
    else:
        print("No predictions at 30%")

    if len(predictions_40) > 0:
        trues = 0
        for each in predictions_40:
            if each == True:
                trues += 1

        print("At 40%:", trues/len(predictions_40), "-",trues,"out of",len(predictions_40))
        returned_for_plots.append([40, trues/len(predictions_40)])
    else:
        print("No predictions at 40%")

    if len(predictions_50) > 0:
        trues = 0
        for each in predictions_50:
            if each == True:
                trues += 1

        print("At 50%:", trues/len(predictions_50), "-",trues,"out of",len(predictions_50))
        returned_for_plots.append([50, trues/len(predictions_50)])
    else:
        print("No predictions at 50%")

    if len(predictions_60) > 0:
        trues = 0
        for each in predictions_60:
            if each == True:
                trues += 1

        print("At 60%:", trues/len(predictions_60), "-",trues,"out of",len(predictions_60))
        returned_for_plots.append([60, trues/len(predictions_60)])
    else:
        print("No predictions at 60%")

    if len(predictions_70) > 0:
        trues = 0
        for each in predictions_70:
            if each == True:
                trues += 1

        print("At 70%:", trues/len(predictions_70), "-",trues,"out of",len(predictions_70))
        returned_for_plots.append([70, trues/len(predictions_70)])
    else:
        print("No predictions at 70%")

    if len(predictions_80) > 0:
        trues = 0
        for each in predictions_80:
            if each == True:
                trues += 1

        print("At 80%:", trues/len(predictions_80), "-",trues,"out of",len(predictions_80))
        returned_for_plots.append([80, trues/len(predictions_80)])
    else:
        print("No predictions at 80%")

    if len(predictions_90) > 0:
        trues = 0
        for each in predictions_90:
            if each == True:
                trues += 1

        print("At 90%:", trues / len(predictions_90), "-", trues, "out of", len(predictions_90))
        returned_for_plots.append([90, trues/len(predictions_90)])
    else:
        print("No predictions at 90%")

    if len(predictions_100) > 0:
        trues = 0
        for each in predictions_100:
            if each == True:
                trues += 1

        print("At 100%:", trues / len(predictions_100), "-", trues, "out of", len(predictions_100))
        returned_for_plots.append([100, trues/len(predictions_100)])
    else:
        print("No predictions at 100%")



    if len(odds_0) > 0:
        trues = 0
        for each in odds_0:
            if each == True:
                trues += 1

        print("At 0%:", trues / len(odds_0), "-", trues, "out of", len(odds_0))
        odds_for_plots.append([0, trues/len(odds_0)])
    else:
        print("No odds at 0%")

    if len(odds_10) > 0:
        trues = 0
        for each in odds_10:
            if each == True:
                trues += 1

        print("At 10%:", trues / len(odds_10), "-", trues, "out of", len(odds_10))
        odds_for_plots.append([10, trues/len(odds_10)])
    else:
        print("No odds at 10%")

    if len(odds_20) > 0:
        trues = 0
        for each in odds_20:
            if each == True:
                trues += 1

        print("At 20%:", trues / len(odds_20), "-", trues, "out of", len(odds_20))
        odds_for_plots.append([20, trues/len(odds_20)])
    else:
        print("No odds at 20%")

    if len(odds_30) > 0:
        trues = 0
        for each in odds_30:
            if each == True:
                trues += 1

        print("At 30%:", trues / len(odds_30), "-", trues, "out of", len(odds_30))
        odds_for_plots.append([30, trues/len(odds_30)])
    else:
        print("No odds at 30%")

    if len(odds_40) > 0:
        trues = 0
        for each in odds_40:
            if each == True:
                trues += 1

        print("At 40%:", trues / len(odds_40), "-", trues, "out of", len(odds_40))
        odds_for_plots.append([40, trues/len(odds_40)])
    else:
        print("No odds at 40%")

    if len(odds_50) > 0:
        trues = 0
        for each in odds_50:
            if each == True:
                trues += 1

        print("At 50%:", trues / len(odds_50), "-", trues, "out of", len(odds_50))
        odds_for_plots.append([50, trues/len(odds_50)])
    else:
        print("No odds at 50%")

    if len(odds_60) > 0:
        trues = 0
        for each in odds_60:
            if each == True:
                trues += 1

        print("At 60%:", trues / len(odds_60), "-", trues, "out of", len(odds_60))
        odds_for_plots.append([60, trues/len(odds_60)])
    else:
        print("No odds at 60%")

    if len(odds_70) > 0:
        trues = 0
        for each in odds_70:
            if each == True:
                trues += 1

        print("At 70%:", trues / len(odds_70), "-", trues, "out of", len(odds_70))
        odds_for_plots.append([70, trues/len(odds_70)])
    else:
        print("No odds at 70%")

    if len(odds_80) > 0:
        trues = 0
        for each in odds_80:
            if each == True:
                trues += 1

        print("At 80%:", trues / len(odds_80), "-", trues, "out of", len(odds_80))
        odds_for_plots.append([80, trues/len(odds_80)])
    else:
        print("No odds at 80%")

    if len(odds_90) > 0:
        trues = 0
        for each in odds_90:
            if each == True:
                trues += 1

        print("At 90%:", trues / len(odds_90), "-", trues, "out of", len(odds_90))
        odds_for_plots.append([90, trues/len(odds_90)])
    else:
        print("No odds at 90%")

    if len(odds_100) > 0:
        trues = 0
        for each in odds_100:
            if each == True:
                trues += 1

        print("At 100%:", trues / len(odds_100), "-", trues, "out of", len(odds_100))
        odds_for_plots.append([100, trues/len(odds_100)])
    else:
        print("No odds at 100%")

    labels_for_matrix = array(labels_for_matrix)
    predictions_for_matrix = array(predictions_for_matrix)
    odds_for_matrix = array(odds_for_matrix)

    return returned_for_plots, odds_for_plots


points_for_plots, odds_for_plots  = run_acc_test(3, "data/test3_cases1_3.csv")
plt.show()

x = []
y = []
for point in points_for_plots:
    x.append(point[0])
    y.append(point[1]*100)

x_opt = [0, 100]
y_opt = [0, 100]

plt.figure()
plt.plot(x_opt, y_opt, linestyle ="--", color="#B7C4DD")
plt.plot(x, y, 'ro')
plt.title("Case 3")
plt.xlabel("Predictions")
plt.ylabel("Actual outcome")
axes = plt.gca()
axes.set_xlim([0,100])
axes.set_ylim([0,100])
plt.xticks([0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100])
plt.yticks([0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100])
plt.show()


x = []
y = []
for point in odds_for_plots:
    x.append(point[0])
    y.append(point[1]*100)

x_opt = [0, 100]
y_opt = [0, 100]

plt.figure()
plt.plot(x_opt, y_opt, linestyle ="--", color="#B7C4DD")
plt.plot(x, y, 'ro')
plt.title("Odds")
plt.xlabel("Predictions")
plt.ylabel("Actual outcome")
axes = plt.gca()
axes.set_xlim([0,100])
axes.set_ylim([0,100])
plt.xticks([0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100])
plt.yticks([0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100])
plt.show()

#Data from Case 4, RNN
plt.figure()
x = [10, 20, 30, 40, 50, 60, 70, 80]
y = [0.11065839207336278, 0.22650200537931223, 0.3241517611308967, 0.39683974356296237, 0.4442898946289594, 0.5294457424076938, 0.6081142183224024, 0.7569530300274145]
y = [i * 100 for i in y]
plt.plot(x_opt, y_opt, linestyle ="--", color="#B7C4DD")
plt.plot(x, y, 'ro')
plt.title("Case 4")
plt.xlabel("Predictions")
plt.ylabel("Actual outcome")
axes = plt.gca()
axes.set_xlim([0,100])
axes.set_ylim([0,100])
plt.xticks([0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100])
plt.yticks([0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100])
plt.show()
