import csv
from datetime import datetime
import numpy as np

def baselines_accuracies():
    training_path = "data/training_cases1_3.csv"

    total_data_cases = 0
    total_correct_home = 0
    total_correct_egrt = 0
    total_correct_xG = 0

    with open(training_path) as f:
        reader = csv.reader(f)

        for row in reader:
            total_data_cases += 1

            #Choose home, or none. Change target
            if row[-1] == "none":
                total_correct_home += 1

            #Choose highest EGRT
            egrt_diff = float(row[1])-float(row[2])
            if egrt_diff > 0:
                if row[-1] == "home":
                    total_correct_egrt+=1
            elif egrt_diff < 0:
                if row[-1] == "away":
                    total_correct_egrt+=1
            else:
                if row[-1] == "home":
                    total_correct_egrt +=1

            #Choose highest xG
            xg_diff = float(row[3])-float(row[4])

            if xg_diff > 0:
                if row[-1] == "home":
                    total_correct_xG+=1
            elif xg_diff < 0:
                if row[-1] == "away":
                    total_correct_xG+=1
            else:
                if row[-1] == "home":
                    total_correct_xG +=1

    print("Home, percentatageg", total_correct_home/total_data_cases)
    print("EGRT, percentatageg", total_correct_egrt/total_data_cases)
    print("xG, percentatageg", total_correct_xG/total_data_cases)

def baselines_rois():
    test_path = "data/test1_cases1_3.csv"
    matches = []
    with open(test_path) as f:
        reader = csv.reader(f)

        for row in reader:

            if row[-1] == "home":
                matches.append([row[-4], [1, 0, 0], float(row[1])-float(row[2]), float(row[3])-float(row[4]), row[0]])
            elif row[-1] == "none":
                matches.append([row[-4], [0, 1, 0], float(row[1])-float(row[2]), float(row[3])-float(row[4]), row[0]])
            elif row[-1] == "away":
                matches.append([row[-4], [0, 0, 1], float(row[1])-float(row[2]), float(row[3])-float(row[4]), row[0]])

    odds_set = {}
    id = "000"
    odds_for_id = []

    with open("odds/uf_lo_export_nextgoal.txt", "r") as oddsfile:
        filereader = csv.reader(oddsfile)

        next(filereader)

        for row in filereader:
            data = row[0].split('\t')
            if id != data[1]:
                odds_set[id] = odds_for_id
                id = data[1]
                odds_for_id = []
            odds_for_id.append(data)
        odds_set[id] = odds_for_id

    total_units_bettet_home = 0
    total_units_bettet_egrt = 0
    total_units_bettet_xg = 0
    total_units_bettet_odds = 0

    bankroll_home = 100
    bankroll_egrt = 100
    bankroll_xg = 100
    bankroll_odds = 100

    for j, match in enumerate(matches):
        match_id = match[-1]
        placing_time = datetime.strptime(match[0], "%Y-%m-%d %H:%M:%S")

        odds_for_match = odds_set[match_id]
        odds_data = odds_for_match[0]
        for each_update in odds_for_match:
            time = datetime.strptime(each_update[2], "%Y-%m-%d %H:%M:%S")
            if time > placing_time:
                break
            odds_data = each_update
        odds = [0.97 / float(odds_data[5]), 0.97 / float(odds_data[7]), 0.97 / float(odds_data[9])]

        index_highest_prob = np.argmin(odds)
        units_to_bet = 1

        total_units_bettet_home += 1
        total_units_bettet_egrt += 1
        total_units_bettet_xg += 1
        total_units_bettet_odds += 1

        bankroll_home -= 1
        bankroll_egrt -= 1
        bankroll_xg -= 1
        bankroll_odds -= 1

        #Bets on home, or none. Change target
        if match[1] ==[0, 1, 0]:
            bankroll_home += units_to_bet*odds[0]

        #Bets highest EGRT
        if match[2] > 0:
            if match[1] == [1, 0, 0]:
                bankroll_egrt += units_to_bet*odds[0]
        elif match[2] < 0:
            if match[1] == [0, 0, 1]:
                bankroll_egrt += units_to_bet*odds[2]
        else:
            if match[1] == [1, 0, 0]:
                bankroll_egrt += units_to_bet*odds[0]

        #Bets best xG
        if match[3] > 0:
            if match[1] == [1, 0, 0]:
                bankroll_xg += units_to_bet*odds[0]
        elif match[3] < 0:
            if match[1] == [0, 0, 1]:
                bankroll_xg += units_to_bet*odds[2]
        else:
            if match[1] == [1, 0, 0]:
                bankroll_xg += units_to_bet*odds[0]

        #Bets lowest odds/highest prob
        if index_highest_prob == 0:
            if match[1] == [1, 0, 0]:
                bankroll_odds += units_to_bet*odds[0]
        elif index_highest_prob == 1:
            if match[1] == [0, 1, 0]:
                bankroll_odds += units_to_bet*odds[1]
        else:
            if match[1] == [0, 0, 1]:
                bankroll_odds += units_to_bet*odds[2]

    print("ROI, fixed bet - home:", ((bankroll_home - 100) / total_units_bettet_home) * 100)
    print("ROI, fixed bet - egrt:", ((bankroll_egrt - 100) / total_units_bettet_egrt) * 100)
    print("ROI, fixed bet - xg:", ((bankroll_xg - 100) / total_units_bettet_xg) * 100)
    print("ROI, fixed bet - odds:", ((bankroll_odds - 100) / total_units_bettet_odds) * 100)

baselines_accuracies()
baselines_rois()
