from tensorflow.keras.models import Sequential
from tensorflow.keras.layers import Dense
from tensorflow.keras import optimizers, losses
from tensorflow.keras import metrics
import tensorflow.keras as keras
from matplotlib import pyplot as plt
import tensorflow as tf
from numpy import array, sum
from tensorflow.keras.models import load_model
from tensorflow.keras.callbacks import EarlyStopping
from tensorflow.keras.callbacks import ModelCheckpoint
import math
import numpy as np
from sklearn.metrics import classification_report, confusion_matrix
import itertools


class PlotLearning(keras.callbacks.Callback):
    #Uncomment all for plots
    best_acc = 0
    best_idx = 0
    def on_train_begin(self, logs={}):
        self.i = 0
        self.x = []
        self.losses = []
        self.val_losses = []
        self.acc = []
        self.val_acc = []
        #self.fig = plt.figure()

        self.logs = []

    def on_epoch_end(self, epoch, logs={}):
        self.logs.append(logs)
        self.x.append(self.i)
        self.losses.append(logs.get('loss'))
        self.val_losses.append(logs.get('val_loss'))
        self.acc.append(logs.get('categorical_accuracy'))
        self.val_acc.append(logs.get('val_categorical_accuracy'))
        self.i += 1

        self.best_idx = self.val_losses.index((min(self.val_losses)))
        self.best_acc = self.val_acc[self.best_idx]

        '''
        f, (ax1, ax2) = plt.subplots(1, 2, sharex=True)
        ax1.plot(self.x, self.losses, label="loss")
        ax1.plot(self.x, self.val_losses, label="val_loss")
        ax1.legend()

        ax2.plot(self.x, self.acc, label="accuracy")
        ax2.plot(self.x, self.val_acc, label="validation accuracy")
        ax2.legend()

        plt.show();
        '''

    def on_train_end(self, logs={}):
        '''
        plt.figure()
        plt.plot(self.x, self.losses, label="loss")
        plt.plot(self.x, self.val_losses, label="val_loss")
        plt.legend()

        plt.figure()
        plt.plot(self.x, self.acc, label="accuracy")
        plt.plot(self.x, self.val_acc, label="validation_accuracy")
        plt.legend()
        '''

class Nn():
    def __init__(self, input_size = 26, output_size = 3, learning_rate = 0.001, hidden_layers = [26], hidden_activation = "relu",
                 optimizer = "adam", output_activation="softmax", epochs=2, batch_size=32, from_file = None):
        self.input_size = input_size
        self.output_size = output_size

        self.learning_rate = learning_rate
        self.hidden_layers = hidden_layers
        self.hidden_activation = hidden_activation
        self.optimizer = optimizer
        self.output_activation = output_activation
        self.epochs = epochs
        self.batch_size = batch_size
        self.accuracy_plots = PlotLearning()

        if from_file:
            self.model = self.load_keras(from_file)
        else:
            self.build()

    def build(self):
        self.model = Sequential()

        self.model.add(Dense(units=self.hidden_layers[0], activation=self.hidden_activation, kernel_initializer="random_uniform",
                             bias_initializer="random_uniform", input_dim=self.input_size))

        if len(self.hidden_layers) > 1:
            for layer in self.hidden_layers[1:]:
                self.model.add(Dense(units=layer, kernel_initializer="random_uniform",
                                     bias_initializer="random_uniform", activation=self.hidden_activation))

        self.model.add(Dense(units=self.output_size, activation=self.output_activation))

        if self.optimizer == "adam":
            optimizer = optimizers.Adam(lr=self.learning_rate)
        elif self.optimizer == "adagrad":
            optimizer = optimizers.Adagrad(lr=self.learning_rate)
        elif self.optimizer == "sgd":
            optimizer = optimizers.SGD(lr=self.learning_rate)
        elif self.optimizer == "rmsprop":
            optimizer = optimizers.RMSprop(lr=self.learning_rate)
        else:
            optimizer = optimizers.Adam(lr=self.learning_rate)

        self.model.compile(loss= losses.categorical_crossentropy,
                           optimizer=optimizer,
                           metrics=[metrics.categorical_accuracy])

    def train(self, cases,  validation_set, best_model_name="nets/not_specified.h5"):

        x_train = []
        y_train = []
        for case in cases:
            x_train.append(case[0])
            y_train.append(case[1])
        x_train = array(x_train)
        y_train = array(y_train)

        x_val = []
        y_val = []
        for case in validation_set:
            x_val.append(case[0])
            y_val.append(case[1])
        x_val = array(x_val)
        y_val = array(y_val)

        es = EarlyStopping(monitor='val_loss', mode='min', verbose=0, patience=5)
        mc = ModelCheckpoint(best_model_name, monitor="val_loss", mode="min", save_best_only=True, verbose=0)

        self.model.fit(x_train, y_train, epochs=self.epochs, validation_data = (x_val, y_val), callbacks=[self.accuracy_plots, es, mc],batch_size=self.batch_size, shuffle=True, verbose=0)
        return self.accuracy_plots.best_acc, self.accuracy_plots.best_idx


    def run_for_betting(self, cases):
        x_test = []

        for case in cases:
            x_test.append(case[0])
        x_test = array(x_test)

        classes = self.model.predict(x_test, batch_size=self.batch_size)

        return classes

    def save_model(self, string):
        self.model.save(string)

    def load_keras(self, name):
        return load_model(name)
